package com.yixin.ms.crypto.util;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.crypto.SecureUtil;
import com.yixin.ms.crypto.crypto.SzhzSm2CryptoKey;
import org.bouncycastle.asn1.gm.*;
import org.bouncycastle.crypto.generators.*;
import java.security.*;
import org.bouncycastle.util.encoders.*;
import org.bouncycastle.asn1.x9.*;
import org.bouncycastle.math.ec.*;
import java.math.*;
import com.alibaba.fastjson.*;
import com.alibaba.fastjson.parser.*;
import cn.superfw.crypto.gmsm.sm2.*;
import org.bouncycastle.crypto.params.*;
import org.bouncycastle.crypto.*;
import org.slf4j.*;

public class SzhzSm2Util {
    private static final Logger log;
    public static final String CRYPTO_NAME_SM2 = "sm2p256v1";
    public static final String SIGN_ID = "1234567812345678";

    public static SzhzSm2CryptoKey getSm2Keys(final boolean compressed) {
        final X9ECParameters sm2ECParameters = GMNamedCurves.getByName("sm2p256v1");
        final ECDomainParameters domainParameters = new ECDomainParameters(sm2ECParameters.getCurve(), sm2ECParameters.getG(), sm2ECParameters.getN());
        final ECKeyPairGenerator keyPairGenerator = new ECKeyPairGenerator();
        try {
            keyPairGenerator.init((KeyGenerationParameters)new ECKeyGenerationParameters(domainParameters, SecureRandom.getInstance("SHA1PRNG")));
        }
        catch (NoSuchAlgorithmException e) {
            SzhzSm2Util.log.error("\u751f\u6210\u516c\u79c1\u94a5\u5bf9\u65f6\u51fa\u73b0\u5f02\u5e38:", (Throwable)e);
        }
        final AsymmetricCipherKeyPair asymmetricCipherKeyPair = keyPairGenerator.generateKeyPair();
        final ECPublicKeyParameters publicKeyParameters = (ECPublicKeyParameters)asymmetricCipherKeyPair.getPublic();
        final ECPoint ecPoint = publicKeyParameters.getQ();
        final String publicKey = Hex.toHexString(ecPoint.getEncoded(compressed));
        final ECPrivateKeyParameters privateKeyParameters = (ECPrivateKeyParameters)asymmetricCipherKeyPair.getPrivate();
        final BigInteger intPrivateKey = privateKeyParameters.getD();
        final String privateKey = intPrivateKey.toString(16);
        SzhzSm2Util.log.debug("\npublicKey\uff1a{}\nprivateKey\uff1a{}", (Object)publicKey, (Object)privateKey);
        return new SzhzSm2CryptoKey(publicKey, privateKey);
    }

    public static <T> String encrypt(final String publicKey, final T content) {
        final String encryptStr = encrypt(publicKey, JSON.toJSONString((Object)content));
        return encryptStr;
    }

    public static <T> T decrypt(final String privateKey, final String content, final Class<T> clazz) {
        final String decryptStr = decrypt(privateKey, content);
        final T t = (T)JSON.parseObject(decryptStr, (TypeReference)new TypeReference<T>() {}, new Feature[0]);
        return t;
    }

    public static String encrypt(final String publicKey, final String data) {
        return encrypt(publicKey, data, SM2EngineExtend.CIPHER_MODE_NORM);
    }

    public static String encrypt(final String publicKey, final String data, final int cipherMode) {
        final X9ECParameters sm2ECParameters = GMNamedCurves.getByName("sm2p256v1");
        final ECDomainParameters domainParameters = new ECDomainParameters(sm2ECParameters.getCurve(), sm2ECParameters.getG(), sm2ECParameters.getN());
        final ECPoint pukPoint = sm2ECParameters.getCurve().decodePoint(Hex.decode(publicKey));
        final ECPublicKeyParameters publicKeyParameters = new ECPublicKeyParameters(pukPoint, domainParameters);
        final SM2EngineExtend sm2Engine = new SM2EngineExtend();
        sm2Engine.init(true, cipherMode, (CipherParameters)new ParametersWithRandom((CipherParameters)publicKeyParameters, new SecureRandom()));
        byte[] arrayOfBytes = null;
        try {
            final byte[] in = data.getBytes();
            arrayOfBytes = sm2Engine.processBlock(in, 0, in.length);
        }
        catch (Exception e) {
            SzhzSm2Util.log.error("SM2\u52a0\u5bc6\u65f6\u51fa\u73b0\u5f02\u5e38:{}", (Object)e.getMessage(), (Object)e);
        }
        return Hex.toHexString(arrayOfBytes);
    }

    public static String decrypt(final String privateKey, final String cipherData) {
        return decrypt(privateKey, cipherData, SM2EngineExtend.CIPHER_MODE_NORM);
    }

    public static String decrypt(final String privateKey, String cipherData, final int cipherMode) {
        if (!cipherData.startsWith("04")) {
            cipherData = "04" + cipherData;
        }
        final byte[] cipherDataByte = Hex.decode(cipherData);
        final X9ECParameters sm2ECParameters = GMNamedCurves.getByName("sm2p256v1");
        final ECDomainParameters domainParameters = new ECDomainParameters(sm2ECParameters.getCurve(), sm2ECParameters.getG(), sm2ECParameters.getN());
        final BigInteger privateKeyD = new BigInteger(privateKey, 16);
        final ECPrivateKeyParameters privateKeyParameters = new ECPrivateKeyParameters(privateKeyD, domainParameters);
        final SM2EngineExtend sm2Engine = new SM2EngineExtend();
        sm2Engine.init(false, cipherMode, (CipherParameters)privateKeyParameters);
        final String result = "";
        try {
            final byte[] arrayOfBytes = sm2Engine.processBlock(cipherDataByte, 0, cipherDataByte.length);
            return new String(arrayOfBytes);
        }
        catch (Exception e) {
            SzhzSm2Util.log.error("SM2\u89e3\u5bc6\u65f6\u51fa\u73b0\u5f02\u5e38:{}", (Object)e.getMessage(), (Object)e);
            return result;
        }
    }

    static {
        log = LoggerFactory.getLogger((Class)SzhzSm2Util.class);
    }
}
