package com.yixin.ms.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.DesensitizedUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.excel.util.CollectionUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.beust.jcommander.internal.Lists;
import com.yixin.ms.boot.restful.RestCode;
import com.yixin.ms.boot.restful.RestResponse;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.boot.uitls.DESUtil;
import com.yixin.ms.boot.uitls.OkHttpUtils;
import com.yixin.ms.boot.uitls.RedisUtil;
import com.yixin.ms.boot.uitls.SMSOrIdCardUtils;
import com.yixin.ms.model.dto.HealthCodeConfigDTO;
import com.yixin.ms.model.dto.HealthCodeDTO;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.enums.*;
import com.yixin.ms.model.vo.HealtCodeNucleinVO;
import com.yixin.ms.model.vo.HealtCodeTypeVO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.HealthCodeServer;
import com.yixin.ms.server.PlaceHealthCodeServer;
import com.yixin.ms.server.dto.DeviceScanCodeDTO;
import com.yixin.ms.server.dto.DeviceScanIdCardDTO;
import com.yixin.ms.server.vo.DeviceScanIdCardVO;
import com.yixin.ms.server.vo.HealthCodeVO;
import com.yixin.ms.server.vo.HealthResultVO;
import com.yixin.ms.server.vo.PlaceInfoVO;
import com.yixin.ms.service.HealthCodeService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Executor;
import java.util.concurrent.TimeUnit;


@Slf4j
@Service
public class HealthCodeServiceImpl implements HealthCodeService {
    /**
     * 设备限制访问标识
     */
    private static final String LIMIT_REQUEST_NUMBER = "health:limit:number:";

    @Autowired
    private RedisUtil redisUtil;
    @Autowired
    private PlaceHealthCodeServer placeHealthCodeServer;
    @Autowired
    private HealthCodeServer healthCodeServer;
    @Autowired
    private RedisTemplate redisTemplate;

    @Override
    public HealtCodeUserVO getShortTermCertificate(ShortTermDTO shortTermDTO) {
        try {
            HealtCodeUserVO shortTermUser = getShortTermUser(shortTermDTO.getIdNumber(), shortTermDTO.getName());
            if (shortTermUser != null && !CollectionUtils.isEmpty(shortTermUser.getData())) {
                return shortTermUser;
            }
            return getSjJkm(shortTermDTO.getIdNumber(), shortTermDTO.getName());
        } catch (Exception e) {
            e.printStackTrace();
            log.info("异常 :" + e.getMessage());
            try {
                return getSjJkm(shortTermDTO.getIdNumber(), shortTermDTO.getName());
            } catch (Exception ex) {
                ex.printStackTrace();
                throw new ServiceException("未获取到健康码信息，请联系工作人员");
            }
        }

    }

    @Override
    public String addToken(String token) {
        String s = "hh_token_" + UUID.randomUUID();
        boolean hanghui = redisUtil.set(s, s);
        if (hanghui) {
            return s;
        } else {
            throw new ServiceException("体检失败");
        }

    }

    @Override
    public Boolean deleteToken(String token) {
        return redisUtil.delete(token);
    }

    @Override
    public RestResult getShortTerm(ShortTermDTO shortTermDTO) {
        log.info("健康码查询入参信息：{}", JSON.toJSONString(shortTermDTO));
        if (StringUtils.isBlank(shortTermDTO.getToken())) {
            throw new ServiceException(700, "token为空哦");
        }

        if(TokenEnum.codeOf(shortTermDTO.getToken()) == null){
            throw new ServiceException(700, "token有误，请检查");
        }

        if (StrUtil.isEmpty(shortTermDTO.getSn())) {
            throw new ServiceException("设备编码必传，请检查入参信息");
        }

        if (StrUtil.isEmpty(shortTermDTO.getHealthCode())) {
            if (StringUtils.isBlank(shortTermDTO.getIdNumber()) && StringUtils.isBlank(shortTermDTO.getName())) {
                throw new ServiceException(701, "身份证和姓名为空哦");
            }
        }

        /**
         * \\000026https://h5.dingtalk.com/healthAct/index.html?stateCouncilBarCode=ffd631d12c5ee80fe77fe0fb4347b4e8&flag=1#/result
         * 扫码参数兼容问题处理
         */
        if (StrUtil.isNotEmpty(shortTermDTO.getHealthCode()) && shortTermDTO.getHealthCode().contains("http")) {
            shortTermDTO.setHealthCode(shortTermDTO.getHealthCode().substring(shortTermDTO.getHealthCode().indexOf("http")));
        }

        if (StrUtil.isNotEmpty(shortTermDTO.getHealthCode()) && shortTermDTO.getHealthCode().contains("HTTP")) {
            shortTermDTO.setHealthCode(shortTermDTO.getHealthCode().substring(shortTermDTO.getHealthCode().indexOf("HTTP")));
        }

        if (StrUtil.isNotEmpty(shortTermDTO.getHealthCode()) && (HttpUtil.isHttp(shortTermDTO.getHealthCode()) || HttpUtil.isHttps(shortTermDTO.getHealthCode()))) {
            log.info("请求健康码参数信息： {}", shortTermDTO.getHealthCode());
            //扫码推送参数为链接，提取杭州健康码barCode参数信息
            String barCode = HttpUtil.decodeParamMap(shortTermDTO.getHealthCode().toLowerCase(), Charset.defaultCharset()).get("statecouncilbarcode");
            if (StrUtil.isEmpty(barCode)) {
                throw new ServiceException("请打开支付宝选择杭州健康码重试");
            } else {
                shortTermDTO.setHealthCode(barCode);
            }
        }

        HealtCodeUserVO userVO = null;
        //核酸有效期时间,默认0不限制
        Integer nucleicExpirationTime = 0;
        RestResult<PlaceInfoVO> placeInfoResult = placeHealthCodeServer.queryOpenPlaceInfoBySn(shortTermDTO.getSn());
        if(placeInfoResult == null){
            throw new ServiceException("浙江服务平台异常，请稍后");
        }

        if(placeInfoResult.getData() == null){
            throw new ServiceException("设备暂无调用权限，请联系管理员");
        }

        if (placeInfoResult != null && placeInfoResult.getData() != null){
            if(placeInfoResult.getData().getRequestNumber().equals(0)){
                throw new ServiceException("设备每秒请求次数限制，请联系管理员");
            }

            String cacheKey = new StringBuilder().append(LIMIT_REQUEST_NUMBER).append(shortTermDTO.getSn()).toString();
            int times = 1;
            Object timesStr = redisTemplate.opsForValue().get(cacheKey);
            if (timesStr == null) {
                //设备限制单位时间访问次数
                redisTemplate.opsForValue().set(cacheKey, times, 1, TimeUnit.SECONDS);
            } else {
                redisTemplate.opsForValue().increment(cacheKey);
                if(Integer.valueOf(timesStr.toString()) > placeInfoResult.getData().getRequestNumber()){
                    throw new ServiceException("设备每秒请求次数限制，请联系管理员");
                }
            }
        }

        if (placeInfoResult != null && placeInfoResult.getData() != null && !placeInfoResult.getData().getEnableStatus() && StrUtil.isNotEmpty(shortTermDTO.getHealthCode())) {
            throw new ServiceException("设备扫码未绑定场所码，请联系管理员");
        }

        if (placeInfoResult != null && placeInfoResult.getData() != null && placeInfoResult.getData().getEnableStatus()) {
            HealthResultVO<DeviceScanIdCardVO> scanIdCardResult = null;
            if (StrUtil.isNotEmpty(shortTermDTO.getHealthCode())) {
                scanIdCardResult = this.deviceScanCode(shortTermDTO);
            } else {
                scanIdCardResult = this.deviceScanIdCardEncrypt(shortTermDTO);
            }

            if (scanIdCardResult == null || !scanIdCardResult.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
                //场所码查询若不成功走之前的
                userVO = this.getShortTermCertificate(shortTermDTO);
            } else {
                DeviceScanIdCardVO scanIdCardVO = scanIdCardResult.getData();
                userVO = new HealtCodeUserVO();
                //疫苗接种数量
                userVO.setVaccineNum(scanIdCardVO.getVaccinated());
                if (StrUtil.isNotEmpty(scanIdCardVO.getCheckTime())) {
                    //核酸记录
                    userVO.setNuclein(Lists.newArrayList(new HealtCodeNucleinVO().setChecktime(DateUtil.parseDateTime(scanIdCardVO.getCheckTime())).setReportTime(StrUtil.isEmpty(scanIdCardVO.getReportTime()) ? null : DateUtil.parseDateTime(scanIdCardVO.getReportTime())).setResult(scanIdCardVO.getCheckResult())));
                } else {
                    userVO.setNuclein(Lists.newArrayList());
                }

                userVO.setData(Lists.newArrayList(new HealtCodeTypeVO().setMzt(HealthCodeLevelEnum.codeOf(scanIdCardVO.getLevel()).getDesc())));
                //行程信息
                userVO.setTravelValidation(scanIdCardVO.getTravelValidation());
                userVO.setUsername(StrUtil.isEmpty(shortTermDTO.getName()) ? scanIdCardVO.getUsername() : shortTermDTO.getName());
            }
        } else {
            //之前的健康码查询
            userVO = this.getShortTermCertificate(shortTermDTO);
        }

        if (userVO != null) {
            if (userVO.getData() == null || userVO.getData().isEmpty()) {
                throw new ServiceException("未查询到健康吗");
            }

            //姓名
            userVO.setUsername(StrUtil.isEmpty(userVO.getUsername()) ? shortTermDTO.getName() : userVO.getUsername());
            //身份证号
            userVO.setIdNumber(StrUtil.isEmpty(userVO.getIdNumber()) ? shortTermDTO.getIdNumber() : userVO.getIdNumber());

            //姓名和身份证号脱敏处理
            userVO.setUsername(StrUtil.isEmpty(userVO.getUsername()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(userVO.getUsername()));
            userVO.setIdNumber(StrUtil.isEmpty(userVO.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(userVO.getIdNumber(), 4, 4));
            /**
             * 0 未接种 对应0针
             *
             * 1-接种完成 对应2针
             *
             * 2-未完全接种 对应1针
             *
             * 3- 完成加强针 对应3针
             */
            if (HealthVaccineNumEnum.VACCINATION_COMPLETE.getStatus().equals(userVO.getVaccineNum())) {
                userVO.setVaccineNum(HealthVaccineNumEnum.VACCINATION_COMPLETE.getNum());
            } else if (HealthVaccineNumEnum.INCOMPLETE_INOCULATION.getStatus().equals(userVO.getVaccineNum())) {
                userVO.setVaccineNum(HealthVaccineNumEnum.INCOMPLETE_INOCULATION.getNum());
            } else {
                userVO.setVaccineNum(userVO.getVaccineNum());
            }

            //用户健康码信息
            HealtCodeTypeVO codeTypeVO = userVO.getData().stream().findFirst().get();
            //用户核酸信息
            List<HealtCodeNucleinVO> nucleinList = userVO.getNuclein();
            if (CollectionUtil.isNotEmpty(nucleinList)) {
                nucleinList.forEach(v -> v.setShowTime(v.getReportTime() == null ? v.getChecktime() : DateUtil.compare(v.getChecktime(), v.getReportTime()) > 0 ? v.getChecktime() : v.getReportTime()));
            }

            if (HealthCodeLevelEnum.GREEN.getDesc().equals(codeTypeVO.getMzt())) {
                codeTypeVO.setContent("绿码：健康码状态为低风险").setContentColor(ColorEnum.GREEN.getValue());
                userVO.setBackgroundColor(ColorEnum.GREEN.getValue());
            }

            //核酸过期标识
            boolean nucleicExpirationFlag = false;
            /**
             * 开门规则判定校验：红黄码，核酸有效期
             */
            if (!HealthCodeLevelEnum.GREEN.getDesc().equals(codeTypeVO.getMzt())) {
                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                String openMsg = new StringBuilder().append(codeTypeVO.getMzt()).append("禁止通行").toString();
                userVO.setOpenMsg(openMsg).setOpenVoiceMsg(openMsg);
                if (HealthCodeLevelEnum.RED.getDesc().equals(codeTypeVO.getMzt())) {
                    codeTypeVO.setContent(new StringBuilder().append(codeTypeVO.getMzt()).append("：健康码状态为高风险").toString()).setContentColor(ColorEnum.RED.getValue());
                    userVO.setBackgroundColor(ColorEnum.RED.getValue());
                } else {
                    codeTypeVO.setContent(new StringBuilder().append(codeTypeVO.getMzt()).append(": 健康码状态为中风险").toString()).setContentColor(ColorEnum.YELLOW.getValue());
                    userVO.setBackgroundColor(ColorEnum.YELLOW.getValue());
                }
            } else if (nucleicExpirationTime != null && !nucleicExpirationTime.equals(0) && (CollectionUtils.isEmpty(nucleinList) || DateUtil.betweenMs(nucleinList.stream().findFirst().get().getShowTime(), DateUtil.date()) > nucleicExpirationTime * 60 * 60 * 1000)) {
                nucleicExpirationFlag = true;
                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                String openMsg;
                if (nucleicExpirationTime <= NucleicRuleEnum.SEVENTY_TWO.getValue()) {
                    openMsg = new StringBuilder().append(nucleicExpirationTime).append("小时内无有效核酸阴性记录，禁止通行").toString();
                } else {
                    openMsg = new StringBuilder().append((nucleicExpirationTime / 24) + 1).append("天内无有效核酸阴性记录，禁止通行").toString();
                }
                userVO.setOpenMsg(openMsg).setOpenVoiceMsg(openMsg);
            } else {
                userVO.setIsOpen(HealthOpenEnum.RELEASE.getCode()).setOpenMsg("绿码，请通行").setOpenVoiceMsg("绿码，请通行");
            }

            if (CollectionUtil.isNotEmpty(nucleinList)) {
                HealtCodeNucleinVO nucleinVO = nucleinList.stream().findFirst().get();
                nucleinVO.setTitle("核酸检测").setContentColor(nucleicExpirationFlag ? ColorEnum.YELLOW.getValue() : ColorEnum.GREEN.getValue()).setIconUrl(nucleicExpirationFlag ? NucleicStatusEnum.NOT_AVAILABLE.getUrl() : NucleicStatusEnum.NEGATIVE.getUrl());
                if (nucleinVO.getReportTime() != null && DateUtil.compare(nucleinVO.getChecktime(), nucleinVO.getReportTime()) > 0) {
                    nucleinVO.setContent(new StringBuilder().append("结果未出").toString());
                } else {
                    long betweenHour = DateUtil.between(nucleinVO.getShowTime(), DateUtil.date(), DateUnit.HOUR);
                    if (betweenHour < NucleicRuleEnum.TWENTY_FOUR.getValue()) {
                        nucleinVO.setContent(new StringBuilder().append(NucleicRuleEnum.TWENTY_FOUR.getValue()).append(" ").append(nucleinVO.getResult()).toString());
                    } else if (betweenHour < NucleicRuleEnum.FORTY_EIGHT.getValue()) {
                        nucleinVO.setContent(new StringBuilder().append(NucleicRuleEnum.FORTY_EIGHT.getValue()).append(" ").append(nucleinVO.getResult()).toString());
                    } else if (betweenHour < NucleicRuleEnum.SEVENTY_TWO.getValue()) {
                        nucleinVO.setContent(new StringBuilder().append(NucleicRuleEnum.SEVENTY_TWO.getValue()).append(" ").append(nucleinVO.getResult()).toString());
                    } else {
                        nucleinVO.setContent(new StringBuilder().append(DateUtil.betweenDay(nucleinVO.getShowTime(), DateUtil.date(), false) + 1).append("天 ").append(nucleinVO.getResult()).toString());
                    }
                }
            } else {
                userVO.setNuclein(Lists.newArrayList(new HealtCodeNucleinVO().setTitle("核酸检测").setIconUrl(NucleicStatusEnum.NOT_AVAILABLE.getUrl()).setContent("暂无").setContentColor(ColorEnum.GREEN.getValue())));
            }

            userVO.setTitle("健 康 码").setTitleColor(ColorEnum.WHITE.getValue())
                    //疫苗信息
                    .setVaccineIcoTitle("新冠疫苗").setVaccineIconUrl(HealthVaccineNumEnum.numOf(userVO.getVaccineNum()).getUrl()).setVaccineNumColor(ColorEnum.GREEN.getValue())
                    //行程信息
                    .setTravelValidationDesc(TravelValidationEnum.codeOf(userVO.getTravelValidation()).getDesc()).setTravelValidationTitle("动态行程卡").setTravelValidationIconUrl(TravelValidationEnum.codeOf(userVO.getTravelValidation()).getUrl());
            return RestResponse.ok(userVO);
        }

        throw new ServiceException(703, "未查询到此数据");
    }

    /**
     * 根据barCode查询健康码信息
     *
     * @param shortTermDTO
     * @return
     */
    private HealthResultVO<DeviceScanIdCardVO> deviceScanCode(ShortTermDTO shortTermDTO) {
        //健康码查询场所码信息
        HealthResultVO<DeviceScanIdCardVO> scanIdCardResult = placeHealthCodeServer.deviceScanCode(new DeviceScanCodeDTO().setDeviceId(shortTermDTO.getSn()).setHealthCodeQrCode(shortTermDTO.getHealthCode()).setUrlParams(shortTermDTO.getHealthCode()));
        //换绑场所码或者管理员健康码首次会提示设备未打卡等错误，重新发起调用
        if (scanIdCardResult == null || !scanIdCardResult.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
            if (scanIdCardResult != null && scanIdCardResult.getCode().equals(Integer.valueOf(RestCode.INVALID_TOKEN.getCode()))) {
                //清理token缓存
                placeHealthCodeServer.getToken(true);
            }

            try {
                Thread.sleep(500);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            scanIdCardResult = placeHealthCodeServer.deviceScanCode(new DeviceScanCodeDTO().setDeviceId(shortTermDTO.getSn()).setHealthCodeQrCode(shortTermDTO.getHealthCode()).setUrlParams(shortTermDTO.getHealthCode()));
        }

        return scanIdCardResult;
    }

    /**
     * 根据证件号查询健康码信息
     *
     * @param shortTermDTO
     * @return
     */
    private HealthResultVO<DeviceScanIdCardVO> deviceScanIdCardEncrypt(ShortTermDTO shortTermDTO) {
        DeviceScanIdCardDTO scanIdCardDTO = new DeviceScanIdCardDTO().setDeviceId(shortTermDTO.getSn()).setIdNumber(shortTermDTO.getIdNumber());
        if (SMSOrIdCardUtils.isIdCard(shortTermDTO.getIdNumber())) {
            scanIdCardDTO.setIdType(IDTypeEnum.ID_CARD.getCode());
        } else {
            scanIdCardDTO.setIdType(IDTypeEnum.PASSPORT.getCode());
        }

        //扫人员证件查询场所码信息
        HealthResultVO<DeviceScanIdCardVO> scanIdCardResult = placeHealthCodeServer.deviceScanIdCardEncrypt(scanIdCardDTO);
        //换绑场所码或者管理员健康码首次会提示设备未打卡等错误，重新发起调用
        if (scanIdCardResult == null || !scanIdCardResult.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
            if (scanIdCardResult != null && scanIdCardResult.getCode().equals(Integer.valueOf(RestCode.INVALID_TOKEN.getCode()))) {
                //清理token缓存
                placeHealthCodeServer.getToken(true);
            }

            try {
                Thread.sleep(500);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            scanIdCardResult = placeHealthCodeServer.deviceScanIdCardEncrypt(scanIdCardDTO);
        }

        return scanIdCardResult;
    }

    @Override
    public RestResult<HealthCodeVO> queryInfoByBarCode(HealthCodeDTO healthCodeDTO) {
        IrsTripAuthEnum authEnum = IrsTripAuthEnum.appKeyOf(healthCodeDTO.getAppKey());
        if (authEnum == null || !authEnum.getAppSecret().equals(healthCodeDTO.getAppSecret())) {
            return RestResponse.error("调用权限校验失败，请检查");
        }

        return healthCodeServer.queryInfoByBarCode(healthCodeDTO.getBarCode());
    }

    @Override
    public RestResult saveHealthCodeConfig(HealthCodeConfigDTO configDTO) {
        if(TokenEnum.codeOf(configDTO.getToken()) == null){
            throw new ServiceException(700, "token有误，请检查");
        }

        return placeHealthCodeServer.saveHealthCodeConfig(configDTO);
    }

    public HealtCodeUserVO getShortTermUser(String sfzh, String name) throws Exception {
        log.info("江干分局健康码传参，身份证{} 姓名 {}", sfzh, name);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("sfzh", sfzh);
        jsonObject.put("xm", name);
        String sync = OkHttpUtils.builder().url("http://115.236.38.205:8903/id/hvn")
                .addHeader("Content-Type", "application/json; charset=utf-8")
                .localPost(jsonObject.toJSONString())
                .sync();
        if (sync.contains("请求失败：Connect timed out")) {
            throw new ServiceException("江干分局请重试");
        }
        log.info("江干分局健康码返回结果，{}", sync);
        HealtCodeUserVO healtCodeUserVO = new HealtCodeUserVO();
        if (StringUtils.isNotBlank(sync)) {
            JSONObject json = JSON.parseObject(sync);
            if (StringUtils.equals("00", json.getString("error_code"))) {
                healtCodeUserVO.setData(JSON.parseArray(json.getJSONArray("data").toJSONString(), HealtCodeTypeVO.class));
                healtCodeUserVO.setNuclein(JSON.parseArray(json.getJSONArray("nuclein").toJSONString(), HealtCodeNucleinVO.class));
                //源数据只返回报告时间，检测时间先等同于报告时间，方便后面逻辑处理
//                healtCodeUserVO.getNuclein().forEach(v -> v.setReportTime(v.getChecktime()));
                healtCodeUserVO.setVaccineNum(json.getString("vaccine_num"));
            } else {
                throw new ServiceException("江干分局异常:" + jsonObject.getString("msg"));
            }
        }
        return healtCodeUserVO;
    }

    public HealtCodeUserVO getSjJkm(String sfzh, String name) throws Exception {
        log.info("市局健康码传参，身份证{} 姓名 {}", sfzh, name);
        String token = "";
        token = (String) redisUtil.get("jkm-token");
        if (StringUtils.isBlank(token)) {
            token = getToken();
            if (StringUtils.isBlank(token)) {
                throw new ServiceException("市局健康码请重试");
            }
        }
        JSONObject data = new JSONObject();
        JSONObject body = new JSONObject();
        body.put("idCard", sfzh);
        String encryptSfzh = DESUtil.encrypt(body.toJSONString(), "7760ce65a8544f779f36607c201e8a0c");
        data.put("body", encryptSfzh);
        data.put("token", token);
        String sync = OkHttpUtils.builder().url("http://policecard.tenzont.com/auth/api/healthCode")
                .addHeader("Content-Type", "application/json; charset=utf-8")
                .localPost(data.toJSONString())
                .sync();
        if (sync.contains("请求失败：Connect timed out")) {
            throw new ServiceException("市局健康码请重试");
        }
        log.info("市局健康码返回结果，{}", sync);
        JSONObject jkm = JSONObject.parseObject(sync);
        String rspCode = jkm.getJSONObject("header").getString("rspCode");
        HealtCodeUserVO healtCodeUserVO = new HealtCodeUserVO();
        if (StringUtils.equals("00", rspCode)) {
            String mzt = jkm.getJSONObject("body").getString("mzt");
            if (StringUtils.isNotBlank(mzt)) {
                List<HealtCodeTypeVO> list = new ArrayList<>();
                HealtCodeTypeVO healtCodeTypeVO = new HealtCodeTypeVO();
                healtCodeTypeVO.setMzt(mzt);
                list.add(healtCodeTypeVO);
                healtCodeUserVO.setData(list);
                healtCodeUserVO.setNuclein(new ArrayList<>());
                healtCodeUserVO.setVaccineNum("");
                return healtCodeUserVO;
            } else {
                throw new ServiceException("市局健康码请重试");
            }
        } else if (StringUtils.equals("01", rspCode)) {
            getToken();
            HealtCodeUserVO sjJkm = getSjJkm(sfzh, name);
            return sjJkm;
        } else {
            throw new ServiceException("市局健康码请重试");
        }
    }


    public String getToken() throws Exception {
        String sync = "";
        JSONObject jsonObject = new JSONObject();
        JSONObject body = new JSONObject();
        body.put("appid", "jzz3c086d786420a958a143e6154a351");
        body.put("secret", "ecbe0e220ec74d05b2b22268a737d2f1");
        jsonObject.put("body", body);
        try {
            sync = OkHttpUtils
                    .builder()
                    .url("http://policecard.tenzont.com/auth/api/token")
                    .localPost(jsonObject.toJSONString())
                    .sync();
        } catch (Exception e) {
            log.info("获取健康码的token接口异常 ，{}", e.getMessage());
            return "";
        }
        if (StringUtils.isBlank(sync)) {
            return "";
        }
        JSONObject fhz = JSONObject.parseObject(sync);
        String rspCode = fhz.getJSONObject("header").getString("rspCode");
        if (StringUtils.equals("00", rspCode)) {
            String token = fhz.getJSONObject("body").getString("token");
            if (StringUtils.isNotBlank(token)) {
                redisUtil.set("jkm-token", token, 7200);
                return token;
            }
        }
        return "";
    }

//    public static void main(String[] args) throws Exception {
//        System.out.println(getSjJkm("412728199706030513", ""));
//    }
}

