package com.yixin.ms.service.impl;

import com.alibaba.fastjson.JSON;
import com.yixin.ms.model.dto.IrsTripCheckScheduleDTO;
import com.yixin.ms.model.dto.IrsTripSendMsgDTO;
import com.yixin.ms.model.enums.IrsTripAuthEnum;
import com.yixin.ms.server.IrsTripServer;
import com.yixin.ms.server.vo.IrsTripResultVO;
import com.yixin.ms.server.vo.IrsTripTokenVO;
import com.yixin.ms.service.IrsTripService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

/**
 * @author: lileilei
 * @date: 2022/8/5 13:48
 * @description: 省共享工信行程
 */
@Slf4j
@Service
public class IrsTripServiceImpl implements IrsTripService {
    @Autowired
    private IrsTripServer irsTripServer;

    @Override
    public String sendMsg(IrsTripSendMsgDTO sendMsgDTO) {
        IrsTripAuthEnum authEnum = IrsTripAuthEnum.appKeyOf(sendMsgDTO.getAppKey());
        if(authEnum == null || !authEnum.getAppSecret().equals(sendMsgDTO.getAppSecret())) {
            return JSON.toJSONString(new IrsTripResultVO<>().setCode(IrsTripResultVO.FAIL_TOKEN_FLAG).setMsg("调用权限校验失败，请检查"));
        }

        IrsTripTokenVO irsTripTokenVO = irsTripServer.getToken();
        log.info("请求token返回内容: {}", JSON.toJSONString(irsTripTokenVO));
        return irsTripServer.sendMsg(sendMsgDTO.getPhone(), irsTripTokenVO.getRequestSecret());
    }

    @Override
    public String checkSchedule(IrsTripCheckScheduleDTO checkScheduleDTO) {
        IrsTripAuthEnum authEnum = IrsTripAuthEnum.appKeyOf(checkScheduleDTO.getAppKey());
        if(authEnum == null || !authEnum.getAppSecret().equals(checkScheduleDTO.getAppSecret())) {
            return JSON.toJSONString(new IrsTripResultVO<>().setCode(IrsTripResultVO.FAIL_TOKEN_FLAG).setMsg("调用权限校验失败，请检查"));
        }

        IrsTripTokenVO irsTripTokenVO = irsTripServer.getToken();
        return irsTripServer.checkSchedule(checkScheduleDTO.getPhone(), checkScheduleDTO.getVerification(), irsTripTokenVO.getRequestSecret(), checkScheduleDTO.getCity());
    }
}
